<?php


namespace Drupal\xi_updater\EventSubscriber;

use Drupal\config_distro\Event\ConfigDistroEvents;
use Drupal\config_merge\Event\ConfigMergeEvent;
use Drupal\config_merge\Event\ConfigMergeEvents;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\xi_updater\ProfileUpdater;
use Symfony\Component\EventDispatcher\Event;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Class ProfileUpdateListener.
 *
 * Respond to event after config from profile was updated.
 *
 * @package Drupal\xi_updater\EventListener
 */
class ProfileUpdateListener implements EventSubscriberInterface {

  use StringTranslationTrait;

  /**
   * @var \Drupal\xi_updater\ProfileUpdater
   */
  protected $profileUpdater;

  /**
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * ProfileUpdateListener constructor.
   *
   * @param \Drupal\xi_updater\ProfileUpdater $profileUpdater
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $loggerChannelFactory
   */
  public function __construct(ProfileUpdater $profileUpdater, LoggerChannelFactoryInterface $loggerChannelFactory) {
    $this->profileUpdater = $profileUpdater;
    $this->logger = $loggerChannelFactory->get('xi_updater');
  }

  /**
   * Registers the methods in this class that should be listeners.
   *
   * @return array
   *   An array of event listener definitions.
   */
  public static function getSubscribedEvents() {
    $events[ConfigDistroEvents::IMPORT][] = ['onConfigDistroImport', 30];
    $events[ConfigMergeEvents::POST_MERGE][] = 'onConfigMerge';
    return $events;
  }

  /**
   * @param \Symfony\Component\EventDispatcher\Event $event
   */
  public function onConfigDistroImport(Event $event) {
    $this->profileUpdater->checkUpdatesStatus();
  }

  /**
   * Reacts on config post merge event.
   *
   * @param \Drupal\config_merge\Event\ConfigMergeEvent $event
   */
  public function onConfigMerge(ConfigMergeEvent $event) {
    $config = $event->getConfigName();
    $provider = [];
    $provider_type = $event->getProviderType();
    if (!empty($provider_type)) {
      $provider[] = $provider_type;
    }
    $provider_name = $event->getProviderName();
    if (!empty($provider_name)) {
      $provider[] = $provider_name;
    }
    $provider = implode(' ', $provider);
    $logs = $event->getLogs();
    foreach ($logs as $operation => $items) {
      $logs = [];
      foreach ($items as $item) {
        if (!in_array($item['name'], $item['parents'])) {
          $item['parents'][] = $item['name'];
        }
        $logs[] = $this->t('@key property change from "@from" to "@to"', [
          '@key' => implode('.', $item['parents']),
          '@from' => is_array($item['state']['previous']) ? implode(', ', $item['state']['previous']) : $item['state']['previous'],
          '@to' => is_array($item['state']['new']) ? implode(', ', $item['state']['new']) : $item['state']['new'],
        ]);
      }
      $message = $this->t('Configuration from @provider in @config was @operation: @list', [
        '@provider' => $provider,
        '@config' => $config,
        '@operation' => $this->t($operation . 'd'),
        '@list' => implode(', ', $logs),
      ]);
      switch ($operation) {
        case 'substitute':
          $log_level = 'warning';
          break;
        case 'ignore':
          $log_level = 'alert';
          break;
        default:
          $log_level = 'info';
          break;
      }
      $this->logger->$log_level($message);
    }
  }

}
