<?php


namespace Drupal\xi_updater;

use Drupal\checklistapi\Storage\StorageInterface;
use Drupal\Core\Entity\EntityStorageException;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\xi_updater\Entity\UpdateInterface;
use Drupal\xi_updater\Reflection\DocTag\Failure;
use Drupal\xi_updater\Reflection\DocTag\Success;
use phpDocumentor\Reflection\DocBlockFactory;
use ReflectionException;
use ReflectionFunction;

/**
 * Class CheckUpdatesStatus
 *
 * Checks the status of the updates, that were performed within the
 * distribution.
 *
 * @package Drupal\xi_updater
 */
class ProfileUpdater {

  /**
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * @var \Drupal\checklistapi\Storage\StorageInterface
   */
  protected $checklistStorage;

  /**
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * CheckUpdatesStatus constructor.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   * @param \Drupal\checklistapi\Storage\StorageInterface $storage
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $loggerChannelFactory
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   */
  public function __construct(EntityTypeManagerInterface $entityTypeManager, StorageInterface $storage, LoggerChannelFactoryInterface $loggerChannelFactory, ModuleHandlerInterface $module_handler) {
    $this->entityTypeManager = $entityTypeManager;
    $this->checklistStorage = $storage;
    $this->checklistStorage->setChecklistId('xi_updater_checklist');
    $this->logger = $loggerChannelFactory->get('xi_updater');
    $this->moduleHandler = $module_handler;
  }

  /**
   * Runs the check of the updates.
   */
  public function checkUpdatesStatus() {
    $update_storage = $this->entityTypeManager->getStorage('xi_update');
    $query = $update_storage->getQuery();
    $query->condition('successful', 0);
    $ids = $query->execute();
    if (!empty($ids)) {
      $updates_to_check = $update_storage->loadMultiple($ids);
      $module = '';
      /** @var \Drupal\xi_updater\Entity\UpdateInterface $update_to_check */
      foreach ($updates_to_check as $update_to_check) {
        if ($module != $update_to_check->getModuleName()) {
          $module = $update_to_check->getModuleName();
          $this->moduleHandler->loadInclude($module, 'install');
        }
        try {
          $this->checkUpdate($update_to_check);
        } catch (EntityStorageException $e) {
          $this->logger->error($e->getMessage());
        }
      }
    }
  }

  /**
   * Checks the status of the update.
   *
   * @param \Drupal\xi_updater\Entity\UpdateInterface $update
   * @param bool $load_install
   *
   * @return bool
   */
  public function checkUpdateStatus(UpdateInterface $update, $load_install = FALSE) {
    if ($load_install) {
      $module = $update->getModuleName();
      $this->moduleHandler->loadInclude($module, 'install');
    }
    $function = $update->id();
    $success = FALSE;
    if (function_exists($function)) {
      $success = $function();
    }
    return (boolean) $success;
  }

  /**
   * @param \Drupal\xi_updater\Entity\UpdateInterface $update
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  private function checkUpdate(UpdateInterface $update) {
    $success = $this->checkUpdateStatus($update);
    if ($success) {
      $update->setSuccessful($success);
      $update->save();
      $progress = $this->checklistStorage->getSavedProgress();
      if (empty($progress)) {
        $progress = [
          '#changed' => time(),
          '#changed_by' => 1,
          '#completed_items' => 1,
        ];
      } else {
        $progress['#changed'] = time();
        $progress['#completed_items']++;
      }
      $progress['#items'][$update->id()] = [
        '#completed' => time(),
        '#uid' => 1,
      ];
      $this->checklistStorage->setSavedProgress($progress);
    }
  }

  /**
   * Creates track of the updates.
   *
   * This method is supposed to run ONLY inside hook_update_N() function,
   * because it relies on the function name of the update hook.
   *
   * @param $function_name
   *
   * @return bool
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function createUpdate($function_name) {
    // Only run on functions which exist.
    if (!function_exists($function_name)) {
      return FALSE;
    }
    $description = 'No description available';
    // The description for an update comes from its Doxygen.
    try {
      [$module, $version] = explode('_update_', $function_name);
      $this->moduleHandler->loadInclude($module, 'install');
      $func = new ReflectionFunction($function_name);
      $description = $func->getDocComment();
    } catch (ReflectionException $e) {
      $this->logger->notice($e->getMessage());
    }
    // Check if this update needs to be tracked.
    if (!$this->isDistroUpdate($description)) {
      return FALSE;
    }
    $update = [
      'id' => $function_name,
      'description' => $description,
      'successful' => FALSE,
    ];
    $storage = $this->entityTypeManager->getStorage('xi_update');
    // Check if update exists already and prevent integrity constraint
    // violation: 1062 Duplicate entry.
    if ($storage->load($function_name) !== NULL) {
      return FALSE;
    }
    $entity = $storage->create($update);
    try {
      $entity->save();
    } catch (EntityStorageException $e) {
      $this->logger->error($e->getMessage());
      return FALSE;
    } finally {
      return TRUE;
    }
  }

  /**
   * Check whether this update is Distro update or not.
   *
   * Distro update should have @success and @failure tags in update function
   * description. In other case it is not the update that needs to be tracked.
   *
   * @param string $description
   *
   * @return bool
   *   TRUE if this update needs to be saved, FALSE in other case.
   */
  private function isDistroUpdate($description) {
    $customTags = [
      'success' => Success::class,
      'failure' => Failure::class,
    ];
    $factory  = DocBlockFactory::createInstance($customTags);
    $docblock = $factory->create($description);
    return $docblock->hasTag('success') && $docblock->hasTag('failure');
  }

  /**
   * Return the item for the checklistapi list.
   *
   * @param \Drupal\xi_updater\Entity\UpdateInterface $update
   *
   * @return array
   */
  public function getChecklistItem(UpdateInterface $update) {
    $customTags = [
      'success' => Success::class,
      'failure' => Failure::class,
    ];
    $description = $update->getDescription();
    $factory  = DocBlockFactory::createInstance($customTags);
    $docblock = $factory->create($description);
    return [
      '#title' => $docblock->getSummary(),
      '#description' => $docblock->getDescription(),
      '#success' => $docblock->hasTag('success') ? $docblock->getTagsByName('success')[0]->__toString() : '',
      '#failure' => $docblock->hasTag('failure') ? $docblock->getTagsByName('failure')[0]->__toString() : '',
    ];
  }

  /**
   * Gets the summary of the update.
   *
   * @param \Drupal\xi_updater\Entity\UpdateInterface $update
   *
   * @return string
   */
  public function getUpdateSummary(UpdateInterface $update) {
    $description = $update->getDescription();
    $factory  = DocBlockFactory::createInstance();
    $docblock = $factory->create($description);
    return $docblock->getSummary();
  }

  /**
   * Gets failure text.
   *
   * @param \Drupal\xi_updater\Entity\UpdateInterface $update
   *
   * @return string
   */
  public function getUpdateFailureText(UpdateInterface $update) {
    $customTags = [
      'failure' => Failure::class,
    ];
    $description = $update->getDescription();
    $factory  = DocBlockFactory::createInstance($customTags);
    $docblock = $factory->create($description);
    return $docblock->hasTag('failure') ? $docblock->getTagsByName('failure')[0]->__toString() : '';
  }

}
